'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useDebouncedCallback;

var _react = require('react');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function useDebouncedCallback(callback, delay, deps) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var maxWait = options.maxWait;

  var maxWaitHandler = (0, _react.useRef)(null);
  var maxWaitArgs = (0, _react.useRef)([]);
  var functionTimeoutHandler = (0, _react.useRef)(null);

  var debouncedFunction = (0, _react.useCallback)(callback, deps);

  var cancelDebouncedCallback = (0, _react.useCallback)(function () {
    clearTimeout(functionTimeoutHandler.current);
    clearTimeout(maxWaitHandler.current);
    maxWaitHandler.current = null;
    maxWaitArgs.current = [];
  }, [functionTimeoutHandler.current, maxWaitHandler.current]);

  (0, _react.useEffect)(function () {
    return function () {
      cancelDebouncedCallback();
    };
  }, []);

  var debouncedCallback = function debouncedCallback() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    maxWaitArgs.current = args;
    clearTimeout(functionTimeoutHandler.current);
    functionTimeoutHandler.current = setTimeout(function () {
      debouncedFunction.apply(undefined, args);

      cancelDebouncedCallback();
    }, delay);

    if (maxWait && !maxWaitHandler.current) {
      maxWaitHandler.current = setTimeout(function () {
        debouncedFunction.apply(undefined, _toConsumableArray(maxWaitArgs.current));
        cancelDebouncedCallback();
      }, maxWait);
    }
  };

  return [debouncedCallback, cancelDebouncedCallback];
}